@@ -1,7 +1,6 @@
-from nexpy.gui.pyqt import QtGui
 import numpy as np
 from nexpy.gui.datadialogs import BaseDialog, GridParameters
-from nexpy.gui.mainwindow import report_error
+from nexpy.gui.utils import report_error
 from nexusformat.nexus import NeXusError
 
 
@@ -16,6 +15,7 @@ def show_dialog(parent=None):
 class EnergyDialog(BaseDialog):
 
     def __init__(self, parent=None):
+
         super(EnergyDialog, self).__init__(parent)
 
         self.select_entry()
