@@ -23,24 +23,24 @@ import logging
 import logging.handlers
 import pkg_resources
 import os
+import shutil
 import signal
 import sys
 import tempfile
 
-from .pyqt import QtCore, QtGui
+from .pyqt import QtCore, QtGui, QtWidgets
 
 from .mainwindow import MainWindow
 from .treeview import NXtree
+from .utils import NXConfigParser, NXLogger, timestamp_age, report_exception
 
-from nexusformat.nexus import nxclasses, nxload
+from nexusformat.nexus import NXroot, nxclasses, nxload, nxversion
 
 from traitlets.config.application import boolean_flag
 from traitlets.config.application import catch_config_error
 from qtconsole.jupyter_widget import JupyterWidget
 from qtconsole.rich_jupyter_widget import RichJupyterWidget
 from qtconsole import styles, __version__
-from qtconsole.client import QtKernelClient
-from qtconsole.manager import QtKernelManager
 from traitlets import (
     Dict, Unicode, CBool, Any
 )
@@ -49,7 +49,10 @@ from jupyter_core.application import JupyterApp, base_flags, base_aliases
 from jupyter_client.consoleapp import (
         JupyterConsoleApp, app_aliases, app_flags,
     )
-from jupyter_client.localinterfaces import is_local_ip
+
+from IPython import __version__ as ipython_version
+from matplotlib import __version__ as mpl_version    
+from .. import __version__ as nexpy_version
 
 #-----------------------------------------------------------------------------
 # Globals
@@ -129,8 +132,6 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
     aliases = Dict(aliases)
     frontend_flags = Any(qt_flags)
     frontend_aliases = Any(qt_aliases)
-    kernel_client_class = QtKernelClient
-    kernel_manager_class = QtKernelManager
 
     stylesheet = Unicode('', config=True,
         help="path to a custom CSS stylesheet")
@@ -139,7 +140,7 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
         help="Start the console window with the menu bar hidden.")
 
     plain = CBool(False, config=True,
-        help="Use a plaintext widget instead of rich text (plain can't print/save).")
+        help="Use a plaintext widget instead of rich text.")
 
     display_banner = CBool(True, config=True,
         help="Whether to display a banner upon starting the QtConsole."
@@ -162,7 +163,7 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
 
     def init_dir(self):
         """Initialize NeXpy home directory"""
-        home_dir = os.path.realpath(os.path.expanduser('~'))
+        home_dir = os.path.abspath(os.path.expanduser('~'))
         nexpy_dir = os.path.join(home_dir, '.nexpy')
         if not os.path.exists(nexpy_dir):
             parent = os.path.dirname(nexpy_dir)
@@ -170,35 +171,75 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
                 nexpy_dir = tempfile.mkdtemp()
             else:
                 os.mkdir(nexpy_dir)
-        for subdirectory in ['functions', 'plugins', 'readers', 'scripts']:
+        for subdirectory in ['backups', 'functions', 'plugins', 'readers', 
+                             'scripts']:
             directory = os.path.join(nexpy_dir, subdirectory)
             if not os.path.exists(directory):
                 os.mkdir(directory)
         global _nexpy_dir
-        _nexpy_dir = nexpy_dir
+        self.nexpy_dir = _nexpy_dir = nexpy_dir
+        self.backup_dir = os.path.join(self.nexpy_dir, 'backups')
+        self.plugin_dir = os.path.join(self.nexpy_dir, 'plugins')
+        self.reader_dir = os.path.join(self.nexpy_dir, 'readers')
+        self.script_dir = os.path.join(self.nexpy_dir, 'scripts')
+        self.function_dir = os.path.join(self.nexpy_dir, 'functions')
+        sys.path.append(self.function_dir)
+        self.scratch_file = os.path.join(self.nexpy_dir, 'w0.nxs')
+        if not os.path.exists(self.scratch_file):
+            NXroot().save(self.scratch_file)
+
+    def init_settings(self):
+        """Initialize access to the NeXpy settings file."""
+        self.settings_file = os.path.join(self.nexpy_dir, 'settings.ini')
+        self.settings = NXConfigParser(self.settings_file)
+        def backup_age(backup):
+            try:
+                return timestamp_age(os.path.basename(os.path.dirname(backup)))
+            except ValueError:
+                return 0
+        backups = self.settings.options('backups')
+        for backup in backups:
+            if not (os.path.exists(backup) and 
+                    os.path.realpath(backup).startswith(self.backup_dir)):
+                self.settings.remove_option('backups', backup)
+            elif backup_age(backup) > 5:
+                shutil.rmtree(os.path.dirname(os.path.realpath(backup)))
+                self.settings.remove_option('backups', backup)
+        self.settings.save()
 
     def init_log(self):
-        value = os.getenv("NEXPY_LOG")
-        if value == None:
-            log_file = os.path.join(_nexpy_dir, 'nexpy.log')
-            hdlr = logging.handlers.RotatingFileHandler(log_file, maxBytes=50000,
-                                                        backupCount=5)
-            fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
-            fmtr = logging.Formatter(fmt, None)
-            logging.root.setLevel(logging.INFO)
+        """Initialize the NeXpy logger."""
+        log_file = os.path.join(self.nexpy_dir, 'nexpy.log')
+        handler = logging.handlers.RotatingFileHandler(log_file, 
+                                                       maxBytes=50000,
+                                                       backupCount=5)
+        fmt = '%(asctime)s - %(levelname)s - %(message)s'
+        formatter = logging.Formatter(fmt, None)
+        handler.setFormatter(formatter)
+        try:
+            if logging.root.hasHandlers():
+                for h in logging.root.handlers:
+                    logging.root.removeHandler(h)
+        except Exception:
+            pass
+        logging.root.addHandler(handler)
+        levels = {'CRITICAL':logging.CRITICAL, 'ERROR':logging.ERROR,
+                  'WARNING':logging.WARNING, 'INFO':logging.INFO, 
+                  'DEBUG':logging.DEBUG}
+        level = os.getenv("NEXPY_LOG")
+        if level is None or level.upper() not in levels:
+            level = 'INFO'
         else:
-            hdlr = logging.StreamHandler(stream=sys.stdout)
-            fmt = '%(levelname)s %(module)s.%(funcName)s() %(message)s'
-            fmtr = logging.Formatter(fmt)
-            try:
-                logging.root.setLevel(logging.__dict__[value])
-            except KeyError:
-                print('Invalid log level:', value)
-                sys.exit(1)
-        hdlr.setFormatter(fmtr)
-        logging.root.addHandler(hdlr)
+            level = level.upper()       
+        logging.root.setLevel(levels[level])
         logging.info('NeXpy launched')
-        logging.debug('Log level is: ' + str(value))
+        logging.info('Log level is ' + level)
+        logging.info('Python ' + sys.version.split()[0] + ': ' + sys.executable)
+        logging.info('IPython v' + ipython_version)
+        logging.info('Matplotlib v' + mpl_version)  
+        logging.info('NeXpy v' + nexpy_version)
+        logging.info('nexusformat v' + nxversion)
+        sys.stdout = sys.stderr = NXLogger()
 
     def init_tree(self):
         """Initialize the NeXus tree used in the tree view."""
@@ -206,16 +247,17 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
         self.tree = NXtree()
         _tree = self.tree
 
+    def init_config(self):
+        self.config.ConsoleWidget.input_sep = ''
+        self.config.Completer.use_jedi = False
+
     def init_gui(self):
         """Initialize the GUI."""
-        self.app = QtGui.QApplication.instance()
+        self.app = QtWidgets.QApplication.instance()
         if self.app is None:
-            self.app = QtGui.QApplication(['nexpy'])
+            self.app = QtWidgets.QApplication(['nexpy'])
         self.app.setApplicationName('nexpy')
-        self.window = MainWindow(self.app, self.tree, config=self.config)
-        self.window.log = self.log
-        global _mainwindow
-        _mainwindow = self.window
+        sys.excepthook = report_exception
         try:
             if 'svg' in QtGui.QImageReader.supportedImageFormats():
                 self.app.icon = QtGui.QIcon(
@@ -225,9 +267,15 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
                 self.app.icon = QtGui.QIcon(
                     pkg_resources.resource_filename('nexpy.gui',
                                                     'resources/icon/NeXpy.png'))
-            QtGui.QApplication.setWindowIcon(self.app.icon)
+            QtWidgets.QApplication.setWindowIcon(self.app.icon)
+            self.icon_pixmap = QtGui.QPixmap(
+                self.app.icon.pixmap(QtCore.QSize(64,64)))
         except Exception:
-            pass
+            self.icon_pixmap = None
+        self.window = MainWindow(self, self.tree, self.settings, self.config)
+        self.window.log = self.log
+        global _mainwindow
+        _mainwindow = self.window
 
     def init_shell(self, filename):
         """Initialize imports in the shell."""
@@ -245,29 +293,32 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
             s = "%s=nx.%s\n" % (_class,_class) + s
         six.exec_(s, self.window.user_ns)
 
-        try:
-            f = open(os.path.join(os.path.expanduser('~'), '.nexpy',
-                                  'config.py'))
-            s = ''.join(f.readlines())
-            six.exec_(s, self.window.user_ns)
-        except:
-            s = ("import sys\n"
-                 "import os\n"
-                 "import h5py as h5\n"
-                 "import numpy as np\n"
-                 "import numpy.ma as ma\n"
-                 "import scipy as sp\n"
-                 "import matplotlib as mpl\n"
-                 "from matplotlib import pylab, mlab, pyplot\n"
-                 "plt = pyplot")
-            six.exec_(s,  self.window.user_ns)
+        config_file = os.path.join(self.nexpy_dir, 'config.py')
+        if not os.path.exists(config_file):
+            s = ["import sys\n",
+                 "import os\n",
+                 "import h5py as h5\n",
+                 "import numpy as np\n",
+                 "import numpy.ma as ma\n",
+                 "import scipy as sp\n",
+                 "import matplotlib as mpl\n",
+                 "from matplotlib import pylab, mlab, pyplot\n",
+                 "plt = pyplot\n",
+                 "os.chdir(os.path.expanduser('~'))\n"]
+            with open(config_file, 'w') as f:
+                f.writelines(s)
+        else:
+            with open(config_file) as f:
+                s = f.readlines()
+        six.exec_('\n'.join(s), self.window.user_ns)
         if filename is not None:
             try:
                 fname = os.path.expanduser(filename)
-                name = _mainwindow.treeview.tree.get_name(fname)
-                _mainwindow.treeview.tree[name] = self.window.user_ns[name] \
+                name = self.window.treeview.tree.get_name(fname)
+                self.window.treeview.tree[name] = self.window.user_ns[name] \
                                                 = nxload(fname)
-                _mainwindow.treeview.select_node(_mainwindow.treeview.tree[name])
+                self.window.treeview.select_node(
+                    self.window.treeview.tree[name])
                 logging.info("NeXus file '%s' opened as workspace '%s'"
                               % (fname, name))
                 self.window.user_ns[name].plot()
@@ -276,10 +327,6 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
 
     def init_colors(self):
         """Configure the coloring of the widget"""
-        # Note: This will be dramatically simplified when colors
-        # are removed from the backend.
-
-        # Configure the style.
         self.window.console.set_default_style()
 
     def init_signal(self):
@@ -299,8 +346,10 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
     def initialize(self, filename=None, argv=None):
         super(NXConsoleApp, self).initialize(argv)
         self.init_dir()
+        self.init_settings()
         self.init_log()
         self.init_tree()
+        self.init_config()
         self.init_gui()
         self.init_shell(filename)
         self.init_colors()
@@ -311,7 +360,6 @@ class NXConsoleApp(JupyterApp, JupyterConsoleApp):
 
         # draw the window
         self.window.show()
-        self.window.raise_()
 
         # Start the application main loop.
         self.app.exec_()
@@ -324,6 +372,7 @@ def main(filename=None):
     app = NXConsoleApp()
     app.initialize(filename=filename)
     app.start()
+    sys.exit(0)
 
 
 if __name__ == '__main__':
