@@ -11,19 +11,25 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import imp
-import logging
+import importlib
 import os
 import re
 import sys
 
-from .pyqt import QtGui, QtCore
 import pkg_resources
 import numpy as np
 
+import matplotlib as mpl
+import lmfit
+
+from .pyqt import QtCore, QtGui, QtWidgets
+
 from nexusformat.nexus import (NeXusError, NXgroup, NXfield, NXattr,
-                               NXroot, NXentry, NXdata, NXparameters)
+                               NXroot, NXentry, NXdata, NXparameters, 
+                               NXnote, NXprocess, nxload)
 from .datadialogs import BaseDialog
+from .plotview import NXPlotView
+from .utils import report_error
 
 from ..api.frills.fit import Fit, Function, Parameter
 
@@ -31,18 +37,21 @@ from ..api.frills.fit import Fit, Function, Parameter
 class FitDialog(BaseDialog):
     """Dialog to fit one-dimensional NeXus data"""
  
-    def __init__(self, entry, parent=None):
+    def __init__(self, entry):
 
-        super(FitDialog, self).__init__(parent)
+        super(FitDialog, self).__init__()
         self.setMinimumWidth(850)        
  
-        self.data = self.initialize_data(entry.data)
+        self._data = self.initialize_data(entry.data)
 
-        from .consoleapp import _tree
-        self.tree = _tree
-
-        from .plotview import plotview
-        self.plotview = plotview
+        if 'Fit' not in self.plotviews:
+            self._fitview = NXPlotView('Fit')
+        self.fitview.plot(self._data, fmt='o')
+        for key in [key for key in mpl.rcParams if key.startswith('keymap')]:
+            for shortcut in 'lr':
+                if shortcut in mpl.rcParams[key]:
+                    mpl.rcParams[key].remove(shortcut)
+        self.fitview.canvas.mpl_connect('key_press_event', self.on_key_press)
         self.functions = []
         self.parameters = []
 
@@ -52,13 +61,14 @@ class FitDialog(BaseDialog):
 
         self.initialize_functions()
  
-        function_layout = QtGui.QHBoxLayout()
-        self.functioncombo = QtGui.QComboBox()
+        function_layout = QtWidgets.QHBoxLayout()
+        self.functioncombo = QtWidgets.QComboBox()
         for name in sorted(self.function_module):
             self.functioncombo.addItem(name)
-        self.functioncombo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        self.functioncombo.setSizeAdjustPolicy(
+            QtWidgets.QComboBox.AdjustToContents)
         self.functioncombo.setMinimumWidth(100)
-        add_button = QtGui.QPushButton("Add Function")
+        add_button = QtWidgets.QPushButton("Add Function")
         add_button.clicked.connect(self.add_function)
         function_layout.addWidget(self.functioncombo)
         function_layout.addWidget(add_button)
@@ -69,34 +79,41 @@ class FitDialog(BaseDialog):
 
         self.parameter_layout = self.initialize_parameter_grid()
 
-        self.remove_layout = QtGui.QHBoxLayout()
-        remove_button = QtGui.QPushButton("Remove Function")
+        self.remove_layout = QtWidgets.QHBoxLayout()
+        remove_button = QtWidgets.QPushButton("Remove Function")
         remove_button.clicked.connect(self.remove_function)
-        self.removecombo = QtGui.QComboBox()
-        self.removecombo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        self.removecombo = QtWidgets.QComboBox()
+        self.removecombo.setSizeAdjustPolicy(
+            QtWidgets.QComboBox.AdjustToContents)
         self.removecombo.setMinimumWidth(100)
         self.remove_layout.addWidget(remove_button)
         self.remove_layout.addWidget(self.removecombo)
         self.remove_layout.addStretch()
 
-        self.plot_layout = QtGui.QHBoxLayout()
-        plot_data_button = QtGui.QPushButton('Plot Data')
+        self.plot_layout = QtWidgets.QHBoxLayout()
+        plot_data_button = QtWidgets.QPushButton('Plot Data')
         plot_data_button.clicked.connect(self.plot_data)
-        plot_function_button = QtGui.QPushButton('Plot Function')
-        plot_function_button.clicked.connect(self.plot_model)
-        self.plotcombo = QtGui.QComboBox()
-        self.plotcombo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        self.plot_function_button = QtWidgets.QPushButton('Plot Function')
+        self.plot_function_button.clicked.connect(self.plot_model)
+        self.plot_function_button.setVisible(False)
+        self.plotcombo = QtWidgets.QComboBox()
+        self.plotcombo.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
         self.plotcombo.setMinimumWidth(100)
-        plot_label = QtGui.QLabel('X-axis:')
-        self.plot_minbox = QtGui.QLineEdit(str(self.plotview.xtab.axis.min))
+        self.plotcombo.setVisible(False)
+        plot_label = QtWidgets.QLabel('X-axis:')
+        self.plot_min = self.fitview.xaxis.min
+        self.plot_max = self.fitview.xaxis.max 
+        self.plot_minbox = QtWidgets.QLineEdit(str(self.plot_min))
         self.plot_minbox.setAlignment(QtCore.Qt.AlignRight)
-        plot_tolabel = QtGui.QLabel(' to ')
-        self.plot_maxbox = QtGui.QLineEdit(str(self.plotview.xtab.axis.max))
+        plot_tolabel = QtWidgets.QLabel(' to ')
+        self.plot_maxbox = QtWidgets.QLineEdit(str(self.plot_max))
         self.plot_maxbox.setAlignment(QtCore.Qt.AlignRight)
-        self.plot_checkbox = QtGui.QCheckBox('Use Data Points')
+        self.plot_checkbox = QtWidgets.QCheckBox('Use Data Points')
+        self.plot_checkbox.setVisible(False)
         self.plot_layout.addWidget(plot_data_button)
-        self.plot_layout.addWidget(plot_function_button)
+        self.plot_layout.addWidget(self.plot_function_button)
         self.plot_layout.addWidget(self.plotcombo)
+        self.plot_layout.addSpacing(5)
         self.plot_layout.addWidget(plot_label)
         self.plot_layout.addWidget(self.plot_minbox)
         self.plot_layout.addWidget(plot_tolabel)
@@ -104,44 +121,63 @@ class FitDialog(BaseDialog):
         self.plot_layout.addWidget(self.plot_checkbox)
         self.plot_layout.addStretch()
 
-        self.action_layout = QtGui.QHBoxLayout()
-        fit_button = QtGui.QPushButton("Fit")
+        self.action_layout = QtWidgets.QHBoxLayout()
+        fit_button = QtWidgets.QPushButton('Fit')
         fit_button.clicked.connect(self.fit_data)
-        self.fit_label = QtGui.QLabel()
-        if self.data.nxerrors:
-            self.fit_checkbox = QtGui.QCheckBox('Use Errors')
+        self.fit_label = QtWidgets.QLabel()
+        if self._data.nxerrors:
+            self.fit_checkbox = QtWidgets.QCheckBox('Use Errors')
             self.fit_checkbox.setCheckState(QtCore.Qt.Checked)
         else:
-            self.fit_checkbox = QtGui.QCheckBox('Use Poisson Errors')
+            self.fit_checkbox = QtWidgets.QCheckBox('Use Poisson Errors')
             self.fit_checkbox.setCheckState(QtCore.Qt.Unchecked)
             self.fit_checkbox.stateChanged.connect(self.define_errors)
-        self.report_button = QtGui.QPushButton("Show Fit Report")
+        self.report_button = QtWidgets.QPushButton("Show Fit Report")
         self.report_button.clicked.connect(self.report_fit)
-        self.save_button = QtGui.QPushButton("Save Parameters")
+        self.save_button = QtWidgets.QPushButton("Save Parameters")
         self.save_button.clicked.connect(self.save_fit)
-        self.restore_button = QtGui.QPushButton("Restore Parameters")
+        self.restore_button = QtWidgets.QPushButton("Restore Parameters")
         self.restore_button.clicked.connect(self.restore_parameters)
         self.action_layout.addWidget(fit_button)
         self.action_layout.addWidget(self.fit_label)
         self.action_layout.addStretch()
         self.action_layout.addWidget(self.fit_checkbox)
+        self.action_layout.addSpacing(5)
         self.action_layout.addWidget(self.save_button)
 
-        self.layout = QtGui.QVBoxLayout()
-        self.layout.addLayout(function_layout)
-        self.layout.addWidget(self.close_buttons())
+        self.bottom_layout = QtWidgets.QHBoxLayout()
+        reset_button = QtWidgets.QPushButton('Reset Limits')
+        reset_button.clicked.connect(self.reset_limits)
+        self.bottom_layout.addWidget(reset_button)
+        self.bottom_layout.addStretch()
+        self.bottom_layout.addWidget(self.close_buttons())
 
+        self.layout = QtWidgets.QVBoxLayout()
+        self.layout.setSpacing(5)
+        self.layout.addLayout(function_layout)
+        self.layout.addLayout(self.plot_layout)
+        self.layout.addLayout(self.bottom_layout)
         self.setLayout(self.layout)
 
         self.setWindowTitle("Fit NeXus Data")
 
         self.load_entry(entry)
 
+    @property
+    def fitview(self):
+        if 'Fit' not in self.plotviews:
+            self._fitview = NXPlotView('Fit')
+        return self.plotviews['Fit']
+
     def initialize_data(self, data):
         if isinstance(data, NXdata):
             if len(data.nxsignal.shape) > 1:
-                raise NeXusError("Fitting only possible on one-dimensional arrays")
-            fit_data = NXdata(data.nxsignal, data.nxaxes, title=data.nxtitle)
+                raise NeXusError(
+                    "Fitting only possible on one-dimensional arrays")
+            signal, axes = data.nxsignal, data.nxaxes[0]
+            if signal.shape[0] == axes.shape[0] - 1:
+                axes = axes.centers()
+            fit_data = NXdata(signal, axes, title=data.nxtitle)
             if data.nxerrors:
                 fit_data.errors = data.nxerrors
             return fit_data
@@ -159,7 +195,8 @@ class FitDialog(BaseDialog):
                 name, ext = os.path.splitext(file_)
                 if name != '__init__' and ext.startswith('.py'):
                     filenames.add(name)
-        functions_path = pkg_resources.resource_filename('nexpy.api.frills', 'functions')
+        functions_path = pkg_resources.resource_filename('nexpy.api.frills', 
+                                                         'functions')
         sys.path.append(functions_path)
         for file_ in os.listdir(functions_path):
             name, ext = os.path.splitext(file_)
@@ -167,28 +204,28 @@ class FitDialog(BaseDialog):
                 filenames.add(name)
         self.function_module = {}
         for name in sorted(filenames):
-            fp, pathname, description = imp.find_module(name)
             try:
-                function_module = imp.load_module(name, fp, pathname, description)
-            finally:
-                if fp:
-                    fp.close()
-            if hasattr(function_module, 'function_name'):
-                self.function_module[function_module.function_name] = function_module
+                function_module = importlib.import_module(name)
+                if hasattr(function_module, 'function_name'):
+                    self.function_module[function_module.function_name] = \
+                        function_module
+            except ImportError:
+                pass
+                
 
     def initialize_parameter_grid(self):
-        grid_layout = QtGui.QVBoxLayout()
-        scroll_area = QtGui.QScrollArea()
+        grid_layout = QtWidgets.QVBoxLayout()
+        scroll_area = QtWidgets.QScrollArea()
         scroll_area.setWidgetResizable(True)
-        scroll_widget = QtGui.QWidget()
+        scroll_widget = QtWidgets.QWidget()
 
-        self.parameter_grid = QtGui.QGridLayout()
-        self.parameter_grid.setSpacing(10)
+        self.parameter_grid = QtWidgets.QGridLayout()
+        self.parameter_grid.setSpacing(5)
         headers = ['Function', 'Np', 'Name', 'Value', '', 'Min', 'Max', 'Fixed']
         width = [100, 50, 100, 100, 100, 100, 100, 50, 100]
         column = 0
         for header in headers:
-            label = QtGui.QLabel()
+            label = QtWidgets.QLabel()
             label.setFont(self.header_font)
             label.setAlignment(QtCore.Qt.AlignHCenter)
             label.setText(header)
@@ -204,11 +241,23 @@ class FitDialog(BaseDialog):
 
         return grid_layout
 
+    @property
+    def data(self):
+        try:
+            xmin, xmax = self.get_limits()
+            axis = self._data.nxaxes[0]
+            if xmin > axis.max() or xmax < axis.min():
+                raise NeXusError('Invalid data range')
+            else:
+                return self._data[xmin:xmax]
+        except NeXusError as error:
+            report_error('Fitting data', error)
+
     def compressed_name(self, name):
-        return re.sub(r'([a-zA-Z]*) # (\d*)', r'\1\2', name)
+        return re.sub(r'([a-zA-Z]*) # (\d*)', r'\1\2', name, count=1)
 
     def expanded_name(self, name):
-        return re.sub(r'([a-zA-Z]*)(\d*)', r'\1 # \2', name)
+        return re.sub(r'([a-zA-Z]*)(\d*)', r'\1 # \2', name, count=1)
     
     def parse_function_name(self, name):
         match = re.match(r'([a-zA-Z]*)(\d*)', name)
@@ -225,8 +274,20 @@ class FitDialog(BaseDialog):
                         if p in entry[group].parameters.entries:
                             parameter = Parameter(p)
                             parameter.value = entry[group].parameters[p].nxdata
-                            parameter.min = float(entry[group].parameters[p].attrs['min'].nxdata)
-                            parameter.max = float(entry[group].parameters[p].attrs['max'].nxdata)                        
+                            parameter.min = float(
+                                entry[group].parameters[p].attrs['min'])
+                            parameter.max = float(
+                                entry[group].parameters[p].attrs['max'])
+                            if 'error' in entry[group].parameters[p].attrs:
+                                error = entry[group].parameters[p].attrs[
+                                            'error']
+                                if error > 0:
+                                    parameter.stderr = float(
+                                        entry[group].parameters[p].attrs[
+                                            'error'])
+                                    parameter.vary = True
+                                else:
+                                    parameter.vary = False
                             parameters.append(parameter)
                     f = Function(group, module, parameters, n)
                     self.functions.append(f)
@@ -243,7 +304,7 @@ class FitDialog(BaseDialog):
         f = Function(name, module, parameters, function_index)
         self.functions.append(f)
         self.index_parameters()
-        self.guess_parameters()
+        self.guess_parameters(f)
         self.add_function_rows(f)
         self.write_parameters()
  
@@ -259,10 +320,12 @@ class FitDialog(BaseDialog):
         if self.first_time:
             self.layout.insertLayout(1, self.parameter_layout)
             self.layout.insertLayout(2, self.remove_layout)
-            self.layout.insertLayout(3, self.plot_layout)
             self.layout.insertLayout(4, self.action_layout)
+            self.plot_function_button.setVisible(True)
             self.plotcombo.addItem('All')
             self.plotcombo.insertSeparator(1)
+            self.plotcombo.setVisible(True)
+            self.plot_checkbox.setVisible(True)
         self.removecombo.addItem(self.expanded_name(f.name))
         self.plotcombo.addItem(self.expanded_name(f.name))
         self.first_time = False
@@ -298,7 +361,8 @@ class FitDialog(BaseDialog):
                 p.parameter_box.setText(str(p.parameter_index))     
 
     def rename_function(self, old_name, new_name):
-        old_name, new_name = self.expanded_name(old_name), self.expanded_name(new_name)
+        old_name, new_name = (self.expanded_name(old_name), 
+                              self.expanded_name(new_name))
         plot_index = self.plotcombo.findText(old_name)
         self.plotcombo.setItemText(plot_index, new_name)
         remove_index = self.removecombo.findText(old_name)
@@ -308,30 +372,28 @@ class FitDialog(BaseDialog):
         row = self.parameter_grid.rowCount()
         name = self.expanded_name(f.name)
         f.rows = []
-        f.label_box = QtGui.QLabel(name)
+        f.label_box = QtWidgets.QLabel(name)
         self.parameter_grid.addWidget(f.label_box, row, 0)
         for p in f.parameters:
             p.parameter_index = row
-            p.parameter_box = QtGui.QLabel(str(p.parameter_index))
-            p.value_box = QtGui.QLineEdit()
+            p.parameter_box = QtWidgets.QLabel(str(p.parameter_index))
+            p.value_box = QtWidgets.QLineEdit()
             p.value_box.setAlignment(QtCore.Qt.AlignRight)
-            p.error_box = QtGui.QLabel()
-            p.min_box = QtGui.QLineEdit('-inf')
+            p.error_box = QtWidgets.QLabel()
+            p.min_box = QtWidgets.QLineEdit('-inf')
             p.min_box.setAlignment(QtCore.Qt.AlignRight)
-            p.max_box = QtGui.QLineEdit('inf')
+            p.max_box = QtWidgets.QLineEdit('inf')
             p.max_box.setAlignment(QtCore.Qt.AlignRight)
-            p.fixed_box = QtGui.QCheckBox()
-#            p.bound_box = QtGui.QLineEdit()
+            p.fixed_box = QtWidgets.QCheckBox()
             self.parameter_grid.addWidget(p.parameter_box, row, 1,
                                           alignment=QtCore.Qt.AlignHCenter)
-            self.parameter_grid.addWidget(QtGui.QLabel(p.name), row, 2)
+            self.parameter_grid.addWidget(QtWidgets.QLabel(p.name), row, 2)
             self.parameter_grid.addWidget(p.value_box, row, 3)
             self.parameter_grid.addWidget(p.error_box, row, 4)
             self.parameter_grid.addWidget(p.min_box, row, 5)
             self.parameter_grid.addWidget(p.max_box, row, 6)
             self.parameter_grid.addWidget(p.fixed_box, row, 7,
                                           alignment=QtCore.Qt.AlignHCenter)
-#            self.parameter_grid.addWidget(p.bound_box, row, 8)
             f.rows.append(row)
             row += 1
         self.parameter_grid.setRowStretch(self.parameter_grid.rowCount(),10)
@@ -350,29 +412,34 @@ class FitDialog(BaseDialog):
                 p.vary = not p.fixed_box.checkState()
 
     def write_parameters(self):
+        def write_value(box, value, prefix=None):
+            try:
+                if prefix:
+                    box.setText('%s %.6g' % (prefix, value))
+                else:
+                    box.setText('%.6g' % value)
+            except TypeError:
+                box.setText(' ')
         for f in self.functions:
             for p in f.parameters:
-                if p.parameter_index:
-                    p.parameter_box.setText(str(p.parameter_index))
-                if p.value:
-                    p.value_box.setText('%.6g' % p.value)
-                if p.vary and p.stderr:
-                    p.error_box.setText('+/- %.6g' % p.stderr)
+                write_value(p.parameter_box, p.parameter_index)
+                write_value(p.value_box, p.value)
+                if p.vary:
+                    write_value(p.error_box, p.stderr, prefix='+/-')
+                write_value(p.min_box, p.min)
+                write_value(p.max_box, p.max)
+                if p.vary:
+                    p.fixed_box.setCheckState(QtCore.Qt.Unchecked)
                 else:
-                    p.error_box.setText(' ')
-                if p.min:
-                    p.min_box.setText('%.6g' % p.min)
-                if p.max:
-                    p.max_box.setText('%.6g' % p.max)
+                    p.fixed_box.setCheckState(QtCore.Qt.Checked)
 
-    def guess_parameters(self):
+    def guess_parameters(self, new_function):
         fit = Fit(self.data, self.functions)
         y = np.array(fit.y)
         for f in self.functions:
-            guess = f.module.guess(fit.x, y)
-            list(map(lambda p, g: p.__setattr__('value', g), f.parameters, 
-                     guess))
-            y = y - f.module.values(fit.x, guess)
+            if f is new_function:
+                f.guess_parameters(fit.x, y)
+            y = y - f.module.values(fit.x, f.parameter_values)
 
     def get_model(self, f=None):
         self.read_parameters()
@@ -380,23 +447,32 @@ class FitDialog(BaseDialog):
         if self.plot_checkbox.isChecked():
             x = fit.x
         else:
-            x = np.linspace(float(self.plot_minbox.text()), 
-                            float(self.plot_maxbox.text()), 1001)
+            xmin, xmax = self.get_limits()
+            x = np.linspace(xmin, xmax, 1001)
         return NXdata(NXfield(fit.get_model(x, f), name='model'),
                       NXfield(x, name=fit.data.nxaxes[0].nxname), 
                       title='Fit Results')
-    
+
+    def get_limits(self):
+        return float(self.plot_minbox.text()), float(self.plot_maxbox.text())
+
+    def reset_limits(self):
+        self.plot_minbox.setText(str(self.plot_min))
+        self.plot_maxbox.setText(str(self.plot_max))
+
     def plot_data(self):
-        self.data.plot()
+        self.fitview.plot(self.data, fmt='o')
+        self.fitview.raise_()
 
     def plot_model(self):
         plot_function = self.plotcombo.currentText()
         if plot_function == 'All':
-            self.get_model().oplot('-')
+            self.fitview.plot(self.get_model(), fmt='-', over=True)
         else:
             name = self.compressed_name(plot_function)
             f = list(filter(lambda x: x.name == name, self.functions))[0]
-            self.get_model(f).oplot('--')
+            self.fitview.plot(self.get_model(f), fmt='--', over=True)
+        self.fitview.raise_()
 
     def define_errors(self):
         if self.fit_checkbox.isChecked():
@@ -408,16 +484,17 @@ class FitDialog(BaseDialog):
             use_errors = True
         else:
             use_errors = False
-        from .mainwindow import report_error
         try:
             self.fit = Fit(self.data, self.functions, use_errors)
             self.fit.fit_data()
         except Exception as error:
             report_error("Fitting Data", error)
         if self.fit.result.success:
-            self.fit_label.setText('Fit Successful Chi^2 = %s' % self.fit.result.redchi)
+            self.fit_label.setText('Fit Successful Chi^2 = %s' 
+                                   % self.fit.result.redchi)
         else:
-            self.fit_label.setText('Fit Failed Chi^2 = %s' % self.fit.result.redchi)
+            self.fit_label.setText('Fit Failed Chi^2 = %s' 
+                                   % self.fit.result.redchi)
         self.write_parameters()
         if not self.fitted:
             self.action_layout.addWidget(self.report_button)
@@ -426,33 +503,26 @@ class FitDialog(BaseDialog):
         self.fitted = True
 
     def report_fit(self):
-        message_box = QtGui.QMessageBox()
+        message_box = QtWidgets.QMessageBox()
         message_box.setText("Fit Results")
-        if self.fit.result.success:
-            summary = 'Fit Successful'
-        else:
-            summary = 'Fit Failed'
         if self.fit.result.errorbars:
             errors = 'Uncertainties estimated'
         else:
             errors = 'Uncertainties not estimated'
-        text = '%s\n' % summary +\
-               '%s\n' % self.fit.result.message +\
-               '%s\n' % self.fit.result.lmdif_message +\
-               'scipy.optimize.leastsq error value = %s\n' % self.fit.result.ier +\
-               'Chi^2 = %s\n' % self.fit.result.chisqr +\
-               'Reduced Chi^2 = %s\n' % self.fit.result.redchi +\
-               '%s\n' % errors +\
-               'No. of Function Evaluations = %s\n' % self.fit.result.nfev +\
-               'No. of Variables = %s\n' % self.fit.result.nvarys +\
-               'No. of Data Points = %s\n' % self.fit.result.ndata +\
-               'No. of Degrees of Freedom = %s\n' % self.fit.result.nfree +\
-               '%s' % self.fit.fit_report()
+        text = ('%s\n' % self.fit.result.message +
+                'Chi^2 = %s\n' % self.fit.result.chisqr +
+                'Reduced Chi^2 = %s\n' % self.fit.result.redchi +
+                '%s\n' % errors +
+                'No. of Function Evaluations = %s\n' % self.fit.result.nfev +
+                'No. of Variables = %s\n' % self.fit.result.nvarys +
+                'No. of Data Points = %s\n' % self.fit.result.ndata +
+                'No. of Degrees of Freedom = %s\n' % self.fit.result.nfree +
+                '%s' % self.fit.fit_report())
         message_box.setInformativeText(text)
-        message_box.setStandardButtons(QtGui.QMessageBox.Ok)
-        spacer = QtGui.QSpacerItem(500, 0, 
-                                   QtGui.QSizePolicy.Minimum, 
-                                   QtGui.QSizePolicy.Expanding)
+        message_box.setStandardButtons(QtWidgets.QMessageBox.Ok)
+        spacer = QtWidgets.QSpacerItem(500, 0, 
+                                   QtWidgets.QSizePolicy.Minimum, 
+                                   QtWidgets.QSizePolicy.Expanding)
         layout = message_box.layout()
         layout.addItem(spacer, layout.rowCount(), 0, 1, layout.columnCount())
         message_box.exec_()
@@ -460,32 +530,41 @@ class FitDialog(BaseDialog):
     def save_fit(self):
         """Saves fit results to an NXentry"""
         self.read_parameters()
-        entry = NXentry()
-        entry['data'] = self.data
+        group = NXprocess()
+        group['data'] = self.data
         for f in self.functions:
-            entry[f.name] = self.get_model(f)
+            group[f.name] = self.get_model(f)
             parameters = NXparameters()
             for p in f.parameters:
                 parameters[p.name] = NXfield(p.value, error=p.stderr, 
                                              initial_value=p.init_value,
                                              min=str(p.min), max=str(p.max))
-            entry[f.name].insert(parameters)
+            group[f.name].insert(parameters)
         if self.fit is not None:
-            entry['title'] = 'Fit Results'
-            entry['fit'] = self.get_model()
+            group['program'] = 'lmfit'
+            group['version'] = lmfit.__version__
+            group['title'] = 'Fit Results'
+            group['fit'] = self.get_model()
             fit = NXparameters()
             fit.nfev = self.fit.result.nfev
-            fit.ier = self.fit.result.ier 
             fit.chisq = self.fit.result.chisqr
             fit.redchi = self.fit.result.redchi
             fit.message = self.fit.result.message
-            fit.lmdif_message = self.fit.result.lmdif_message
-            entry['statistics'] = fit
+            group['statistics'] = fit
+            group.note = NXnote(self.fit.result.message,
+                ('Chi^2 = %s\n' % self.fit.result.chisqr +
+                 'Reduced Chi^2 = %s\n' % self.fit.result.redchi +
+
+                 'No. of Function Evaluations = %s\n' % self.fit.result.nfev +
+                 'No. of Variables = %s\n' % self.fit.result.nvarys +
+                 'No. of Data Points = %s\n' % self.fit.result.ndata +
+                 'No. of Degrees of Freedom = %s\n' % self.fit.result.nfree +
+                 '%s' % self.fit.fit_report()))
         else:
-            entry['title'] = 'Fit Model'
-            entry['model'] = self.get_model()
+            group['title'] = 'Fit Model'
+            group['model'] = self.get_model()
         if 'w0' not in self.tree:
-            self.tree.add(NXroot(name='w0'))
+            self.tree['w0'] = nxload(self.mainwindow.scratch_file, 'rw')
         ind = []
         for key in self.tree['w0']:
             try:
@@ -496,7 +575,7 @@ class FitDialog(BaseDialog):
         if not ind:
             ind = [0]
         name = 'f'+str(sorted(ind)[-1]+1)
-        self.tree['w0'][name] = entry
+        self.tree['w0'][name] = group
 
     def restore_parameters(self):
         for f in self.functions:
@@ -505,12 +584,25 @@ class FitDialog(BaseDialog):
                 p.stderr = None
         self.fit_label.setText(' ')
         self.write_parameters()
+
+    def on_key_press(self, event):
+        if event.inaxes:
+            if event.key == 'l':
+                self.plot_minbox.setText('%g' % event.xdata)
+            elif event.key == 'r':
+                self.plot_maxbox.setText('%g' % event.xdata)
    
     def accept(self):
+        if 'Fit' in self.plotviews:
+            self.fitview.close()
         super(FitDialog, self).accept()
         
     def reject(self):
+        if 'Fit' in self.plotviews:
+            self.fitview.close()
         super(FitDialog, self).reject()
 
     def closeEvent(self, event):
+        if 'Fit' in self.plotviews:
+            self.fitview.close()
         event.accept()
