@@ -18,9 +18,9 @@ import tempfile
 import pygments
 from pygments.formatter import Formatter
 
-from .pyqt import QtCore, QtGui, getSaveFileName
+from .pyqt import QtCore, QtGui, QtWidgets, getSaveFileName
 
-from .datadialogs import BaseDialog
+from .utils import confirm_action
 
 
 def hex2QColor(c):
@@ -39,7 +39,7 @@ class QFormatter(Formatter):
         
         self.styles={}
         for token, style in self.style:
-            qtf=QtGui.QTextCharFormat()
+            qtf = QtGui.QTextCharFormat()
 
             if style['color']:
                 qtf.setForeground(hex2QColor(style['color'])) 
@@ -54,7 +54,6 @@ class QFormatter(Formatter):
             self.styles[str(token)] = qtf
     
     def format(self, tokensource, outfile):
-        global styles
         self.data=[]
         for ttype, value in tokensource:
             l=len(value)
@@ -86,8 +85,7 @@ class Highlighter(QtGui.QSyntaxHighlighter):
                 pass
 
 
-
-class NXPlainTextEdit(QtGui.QPlainTextEdit):
+class NXPlainTextEdit(QtWidgets.QPlainTextEdit):
 
     def __init__(self, parent):
         super(NXPlainTextEdit, self).__init__()
@@ -120,12 +118,12 @@ class NXPlainTextEdit(QtGui.QPlainTextEdit):
                    self.line_height)
 
        
-class NXScriptWindow(QtGui.QDialog):
+class NXScriptWindow(QtWidgets.QDialog):
 
     def __init__(self, parent=None):
         super(NXScriptWindow, self).__init__(parent)
-        layout = QtGui.QVBoxLayout()
-        self.tabs = QtGui.QTabWidget(self)
+        layout = QtWidgets.QVBoxLayout()
+        self.tabs = QtWidgets.QTabWidget(self)
         layout.addWidget(self.tabs)
         self.setLayout(layout)
         self.setWindowTitle('Script Editor')
@@ -138,6 +136,10 @@ class NXScriptWindow(QtGui.QDialog):
     def editors(self):
         return [self.tabs.widget(idx) for idx in range(self.tabs.count())]
 
+    @property
+    def editor(self):
+        return self.tabs.currentWidget()
+
     def update(self):
         for editor in self.editors:
             editor.adjustSize()
@@ -149,29 +151,32 @@ class NXScriptWindow(QtGui.QDialog):
         event.accept()
         
     def close(self):
-        for editor in self.editors:
-            editor.close()
-        self.setVisible(False)
+        self.editor.close()
+        if self.tabs.count() == 0:
+            self.setVisible(False)
 
 
-class NXScriptEditor(QtGui.QWidget):
+class NXScriptEditor(QtWidgets.QWidget):
     """Dialog to plot arbitrary NeXus data in one or two dimensions"""
  
     def __init__(self, file_name=None, parent=None):
 
-        from .consoleapp import _mainwindow
-        self.window = _mainwindow.editors
+        if parent is None:
+            from .consoleapp import _mainwindow
+            self.mainwindow = _mainwindow
+        else:
+            self.mainwindow = parent
+        self.window = self.mainwindow.editors
 
-        QtGui.QWidget.__init__(self, parent=self.window.tabs)
+        QtWidgets.QWidget.__init__(self, parent=self.window.tabs)
  
         self.file_name = file_name
-        from .consoleapp import _nexpy_dir
-        self.default_directory = os.path.join(_nexpy_dir, 'scripts')
+        self.default_directory = self.mainwindow.script_dir
 
-        layout = QtGui.QVBoxLayout()
-        self.text_layout = QtGui.QHBoxLayout()
+        layout = QtWidgets.QVBoxLayout()
+        self.text_layout = QtWidgets.QHBoxLayout()
         if sys.platform == 'darwin':
-            self.number_box = QtGui.QLabel('1')
+            self.number_box = QtWidgets.QLabel('1')
             self.number_box.setFont(QtGui.QFont('Courier'))
             self.number_box.setAlignment(QtCore.Qt.AlignTop | 
                                          QtCore.Qt.AlignRight)
@@ -181,20 +186,20 @@ class NXScriptEditor(QtGui.QWidget):
         self.text_layout.addWidget(self.text_box)
         layout.addLayout(self.text_layout)
         
-        run_button = QtGui.QPushButton('Run Script')
+        run_button = QtWidgets.QPushButton('Run Script')
         run_button.clicked.connect(self.run_script)
         run_button.setAutoDefault(False)
-        self.argument_box = QtGui.QLineEdit()
+        self.argument_box = QtWidgets.QLineEdit()
         self.argument_box.setMinimumWidth(200)
-        save_button = QtGui.QPushButton('Save')
+        save_button = QtWidgets.QPushButton('Save')
         save_button.clicked.connect(self.save_script)
-        save_as_button = QtGui.QPushButton('Save as...')
+        save_as_button = QtWidgets.QPushButton('Save as...')
         save_as_button.clicked.connect(self.save_script_as)
-        self.delete_button = QtGui.QPushButton('Delete')
+        self.delete_button = QtWidgets.QPushButton('Delete')
         self.delete_button.clicked.connect(self.delete_script)
-        close_button = QtGui.QPushButton('Close Tab')
+        close_button = QtWidgets.QPushButton('Close Tab')
         close_button.clicked.connect(self.close)
-        button_layout = QtGui.QHBoxLayout()
+        button_layout = QtWidgets.QHBoxLayout()
         button_layout.addWidget(run_button)
         button_layout.addWidget(self.argument_box)
         button_layout.addStretch()
@@ -227,7 +232,7 @@ class NXScriptEditor(QtGui.QWidget):
         return 'NXScriptEditor(%s)' % self.label
         
     def get_text(self):
-        return self.text_box.document().toPlainText()+'\n'
+        return self.text_box.document().toPlainText().strip()+'\n'
 
     def update_line_numbers(self, count):
         if sys.platform != 'darwin':
@@ -246,17 +251,16 @@ class NXScriptEditor(QtGui.QWidget):
             self.number_box.setStyleSheet("QLabel {padding: 1px 0}")
 
     def run_script(self):
-        from .consoleapp import _mainwindow
         text = self.get_text()
         if 'sys.argv' in text:
             file_name = tempfile.mkstemp('.py')[1]
             with open(file_name, 'w') as f:
                 f.write(self.get_text())
             args = self.argument_box.text()
-            _mainwindow.console.execute('run -i %s %s' % (file_name, args))
+            self.mainwindow.console.execute('run -i %s %s' % (file_name, args))
             os.remove(file_name)
         else:
-            _mainwindow.console.execute(self.get_text())
+            self.mainwindow.console.execute(self.get_text())
 
     def save_script(self):
         if self.file_name:
@@ -273,32 +277,20 @@ class NXScriptEditor(QtGui.QWidget):
             with open(file_name, 'w') as f:
                 f.write(self.get_text())
             self.file_name = file_name
-            self.window.tabs.setTabText(self.index, os.path.basename(self.file_name))
-            from .consoleapp import _mainwindow
-            _mainwindow.add_script_action(self.file_name)
+            self.window.tabs.setTabText(self.index, 
+                                        os.path.basename(self.file_name))
+            self.mainwindow.add_script_action(self.file_name)
             self.delete_button.setVisible(True)
 
     def delete_script(self):
         if self.file_name:
-            ret = self.confirm_action(
-                      "Are you sure you want to delete '%s'?" % self.file_name,
-                      "This cannot be reversed")
-            if ret == QtGui.QMessageBox.Ok:
+            if confirm_action(
+                    "Are you sure you want to delete '%s'?" % self.file_name,
+                    "This cannot be reversed"):
                 os.remove(self.file_name)
-                from .consoleapp import _mainwindow
-                _mainwindow.remove_script_action(self.file_name)
+                self.mainwindow.remove_script_action(self.file_name)
                 self.close()
 
-    def confirm_action(self, query, information=None):
-        message_box = QtGui.QMessageBox()
-        message_box.setText(query)
-        if information:
-            message_box.setInformativeText(information)
-        message_box.setStandardButtons(QtGui.QMessageBox.Ok | 
-                                       QtGui.QMessageBox.Cancel)
-        message_box.setDefaultButton(QtGui.QMessageBox.Ok)
-        return message_box.exec_()
-
     def close(self):
         self.window.tabs.removeTab(self.index)
         self.deleteLater()
