@@ -19,8 +19,8 @@ import os
 import re
 import numpy as np
 
-from nexusformat.nexus import NXfield, NXentry, NXdata, NeXusError
-from nexpy.gui.pyqt import QtGui, QtCore
+from nexusformat.nexus import *
+from nexpy.gui.pyqt import QtCore, QtWidgets
 from nexpy.gui.importdialog import BaseImportDialog
 
 filetype = "Image Stack"
@@ -29,13 +29,13 @@ prefix_pattern = re.compile('^([^.]+)(?:(?<!\d)|(?=_))')
 
 
 class ImportDialog(BaseImportDialog):
-    """Dialog to import an image stack (TIFF or CBF)"""
+    """Dialog to import an image stack using Fabio"""
  
     def __init__(self, parent=None):
 
         super(ImportDialog, self).__init__(parent)
         
-        self.layout = QtGui.QVBoxLayout()
+        self.layout = QtWidgets.QVBoxLayout()
 
         self.layout.addLayout(self.directorybox())
 
@@ -45,8 +45,8 @@ class ImportDialog(BaseImportDialog):
         self.rangebox = self.make_rangebox()
         self.layout.addWidget(self.rangebox)
 
-        status_layout = QtGui.QHBoxLayout()
-        self.progress_bar = QtGui.QProgressBar()
+        status_layout = QtWidgets.QHBoxLayout()
+        self.progress_bar = QtWidgets.QProgressBar()
         status_layout.addWidget(self.progress_bar)
         self.progress_bar.setVisible(False)
         status_layout.addStretch()
@@ -62,17 +62,17 @@ class ImportDialog(BaseImportDialog):
         return self.suffix_box.text()
 
     def make_filterbox(self):
-        filterbox = QtGui.QWidget()
-        layout = QtGui.QGridLayout()
+        filterbox = QtWidgets.QWidget()
+        layout = QtWidgets.QGridLayout()
         layout.setSpacing(10)
-        prefix_label = QtGui.QLabel('File Prefix')
-        self.prefix_box = QtGui.QLineEdit()
+        prefix_label = QtWidgets.QLabel('File Prefix')
+        self.prefix_box = QtWidgets.QLineEdit()
         self.prefix_box.editingFinished.connect(self.set_range)
-        suffix_label = QtGui.QLabel('File Suffix')
-        self.suffix_box = QtGui.QLineEdit('')
+        suffix_label = QtWidgets.QLabel('File Suffix')
+        self.suffix_box = QtWidgets.QLineEdit('')
         self.suffix_box.editingFinished.connect(self.get_prefixes)
-        extension_label = QtGui.QLabel('File Extension')
-        self.extension_box = QtGui.QLineEdit()
+        extension_label = QtWidgets.QLabel('File Extension')
+        self.extension_box = QtWidgets.QLineEdit()
         self.extension_box.editingFinished.connect(self.set_extension)
         layout.addWidget(prefix_label, 0, 0)
         layout.addWidget(self.prefix_box, 0, 1)
@@ -80,27 +80,31 @@ class ImportDialog(BaseImportDialog):
         layout.addWidget(self.suffix_box, 0, 3)
         layout.addWidget(extension_label, 0, 4)
         layout.addWidget(self.extension_box, 0, 5)
-        self.prefix_combo = QtGui.QComboBox()
-        self.prefix_combo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        self.prefix_combo = QtWidgets.QComboBox()
+        self.prefix_combo.setSizeAdjustPolicy(
+            QtWidgets.QComboBox.AdjustToContents)
         self.prefix_combo.activated.connect(self.choose_prefix)
-        self.extension_combo = QtGui.QComboBox()
-        self.extension_combo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        self.extension_combo = QtWidgets.QComboBox()
+        self.extension_combo.setSizeAdjustPolicy(
+            QtWidgets.QComboBox.AdjustToContents)
         self.extension_combo.activated.connect(self.choose_extension)
-        layout.addWidget(self.prefix_combo, 1, 1, alignment=QtCore.Qt.AlignHCenter)
-        layout.addWidget(self.extension_combo, 1, 5, alignment=QtCore.Qt.AlignHCenter)
+        layout.addWidget(self.prefix_combo, 1, 1, 
+                         alignment=QtCore.Qt.AlignHCenter)
+        layout.addWidget(self.extension_combo, 1, 5, 
+                         alignment=QtCore.Qt.AlignHCenter)
         filterbox.setLayout(layout)
         filterbox.setVisible(False)
         return filterbox
 
     def make_rangebox(self):
-        rangebox = QtGui.QWidget()
-        layout = QtGui.QHBoxLayout()
-        rangeminlabel = QtGui.QLabel("Min. index")
-        self.rangemin = QtGui.QLineEdit()
+        rangebox = QtWidgets.QWidget()
+        layout = QtWidgets.QHBoxLayout()
+        rangeminlabel = QtWidgets.QLabel("Min. index")
+        self.rangemin = QtWidgets.QLineEdit()
         self.rangemin.setFixedWidth(150)
         self.rangemin.setAlignment(QtCore.Qt.AlignRight)
-        rangemaxlabel = QtGui.QLabel("Max. index")
-        self.rangemax = QtGui.QLineEdit()
+        rangemaxlabel = QtWidgets.QLabel("Max. index")
+        self.rangemax = QtWidgets.QLineEdit()
         self.rangemax.setFixedWidth(150)
         self.rangemax.setAlignment(QtCore.Qt.AlignRight)
         layout.addWidget(rangeminlabel)
@@ -133,7 +137,8 @@ class ImportDialog(BaseImportDialog):
                 self.prefix_combo.addItem(prefix)
         if self.get_prefix() not in prefixes:
             self.set_prefix(prefixes[0])
-        self.prefix_combo.setCurrentIndex(self.prefix_combo.findText(self.get_prefix()))
+        self.prefix_combo.setCurrentIndex(
+            self.prefix_combo.findText(self.get_prefix()))
         try:
             files = [f for f in files if f.startswith(self.get_prefix())]
             min, max = self.get_index(files[0]), self.get_index(files[-1])
@@ -170,7 +175,10 @@ class ImportDialog(BaseImportDialog):
                 self.set_extension('.tiff')
             elif '.cbf' in extensions:
                 self.set_extension('.cbf')
-        self.extension_combo.setCurrentIndex(self.extension_combo.findText(self.get_extension()))
+            else:
+                self.set_extension(list(extensions)[0])
+        self.extension_combo.setCurrentIndex(
+            self.extension_combo.findText(self.get_extension()))
         return extensions
 
     def get_extension(self):
@@ -187,17 +195,13 @@ class ImportDialog(BaseImportDialog):
             text = '.'+text
         self.extension_box.setText(text)
         if self.extension_combo.findText(text) >= 0:
-            self.extension_combo.setCurrentIndex(self.extension_combo.findText(text))
+            self.extension_combo.setCurrentIndex(
+                self.extension_combo.findText(text))
         self.get_prefixes()
 
-    def get_image_type(self):
-        if self.get_extension() == '.cbf':
-            return 'CBF'
-        else:
-            return 'TIFF'
-
     def get_index(self, file):
-        return int(re.match('^(.*?)([0-9]*)%s[.](.*)$' % self.suffix, file).groups()[1])
+        return int(re.match('^(.*?)([0-9]*)%s[.](.*)$' % self.suffix, 
+                   file).groups()[1])
 
     def get_indices(self):
         try:
@@ -217,13 +221,15 @@ class ImportDialog(BaseImportDialog):
                                               self.get_extension())
         if self.get_indices():
             min, max = self.get_indices()
-            return [file for file in filenames if self.get_index(file) >= min and 
-                                                  self.get_index(file) <= max]
+            return [file for file in filenames 
+                    if self.get_index(file) >= min and 
+                    self.get_index(file) <= max]
         else:
             return filenames
 
     def set_range(self):
-        files = self.get_filesindirectory(self.get_prefix(), self.get_extension())
+        files = self.get_filesindirectory(self.get_prefix(), 
+                                          self.get_extension())
         try:
             min, max = self.get_index(files[0]), self.get_index(files[-1])
             if min > max:
@@ -235,24 +241,18 @@ class ImportDialog(BaseImportDialog):
             self.rangebox.setVisible(False)
 
     def read_image(self, filename):
-        if self.get_image_type() == 'CBF':
-            import pycbf
-            cbf = pycbf.cbf_handle_struct()
-            cbf.read_file(str(filename), pycbf.MSG_DIGEST)
-            cbf.select_datablock(0)
-            cbf.select_category(0)
-            cbf.select_column(2)
-            imsize = cbf.get_image_size(0)
-            return np.fromstring(cbf.get_integerarray_as_string(),np.int32).reshape(imsize)
-        else:
-            import tifffile
-            return tifffile.imread(filename)
+        try:
+            import fabio
+        except ImportError:
+            raise NeXusError("Please install the 'fabio' module")
+        im = fabio.open(filename)
+        return im
 
     def read_images(self, filenames):
-        v0 = self.read_image(filenames[0])
+        v0 = self.read_image(filenames[0]).data
         v = np.zeros([len(filenames), v0.shape[0], v0.shape[1]], dtype=np.int32)
         for i,filename in enumerate(filenames):
-            v[i] = self.read_image(filename)
+            v[i] = self.read_image(filename).data
         global maximum
         if v.max() > maximum:
             maximum = v.max()
@@ -265,17 +265,12 @@ class ImportDialog(BaseImportDialog):
         else:
             self.import_file = self.get_directory()       
         filenames = self.get_files()
-        if self.get_image_type() == 'CBF':
-            try:
-                import pycbf
-            except ImportError:
-                raise NeXusError("Please install the 'pycbf' module")
-        else:
-            try:
-                import tifffile
-            except ImportError:
-                raise NeXusError("Please install the 'tifffile' module")
-        v0 = self.read_image(filenames[0])
+        try:
+            import fabio
+        except ImportError:
+            raise NeXusError("Please install the 'fabio' module")
+        im = self.read_image(filenames[0])
+        v0 = im.data
         x = NXfield(range(v0.shape[1]), dtype=np.uint16, name='x')
         y = NXfield(range(v0.shape[0]), dtype=np.uint16, name='y')
         z = NXfield(range(1,len(filenames)+1), dtype=np.uint16, name='z')
@@ -300,4 +295,20 @@ class ImportDialog(BaseImportDialog):
                 pass
         global maximum
         v.maximum = maximum
-        return NXentry(NXdata(v,(z,y,x)))
\ No newline at end of file
+
+        if im.getclassname() == 'CbfImage':
+            note = NXnote(type='text/plain', file_name=self.import_file)
+            note.data = im.header.pop('_array_data.header_contents', '')
+            note.description = im.header.pop(
+                '_array_data.header_convention', '')
+        else:
+            note = None
+
+        header = NXcollection()
+        for key, value in im.header.items():
+            header[key] = value
+
+        if note:
+            return NXentry(NXdata(v,(z,y,x), CBF_header=note, header=header))
+        else:
+            return NXentry(NXdata(v,(z,y,x), header=header))
