@@ -3,7 +3,7 @@
 '''package requirements are checked at runtime and installation time'''
 
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013-2014, NeXpy Development Team.
+# Copyright (c) 2013-2016, NeXpy Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -11,14 +11,16 @@
 #-----------------------------------------------------------------------------
 
 pkg_requirements = [
-    'nexusformat>=0.2.1',
+    'nexusformat>=0.4.18',
     'numpy>=1.6.0',
     'scipy',
     'h5py',
     'jupyter',
     'ipython>=4.0.0',
-    'matplotlib>=1.4.0',
+    'matplotlib>=1.5.0',
+    'ansi2html',
+    'pillow'
 ]
 extra_requirements = {
-    'spec': ['spec2nexus>=2014.1228.0',],
+    'spec': ['spec2nexus>=2017.901.4',],
 }
